/* pickOne.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "pickOne.h"

PickOne::
PickOne() {
  _has_item = false;
  _current_item = 0;
  _internal_change = false;
  set_value_in_list(true, false);

  connect_to_method(get_entry()->changed, this, &entry_changed);
}

PickOne::
~PickOne() {
  // We set our text to the empty string, because otherwise Gtk seems
  // to get confused when we destruct (if our text did not match any
  // of the choices).
  _internal_change = true;
  get_entry()->set_text("");
}

void PickOne::
set_choices(const Choices &choices) {
  _choices = choices;
  finish_choices();
}

void PickOne::
add_choice(int code, const string &text) {
  _choices.push_back(Choices::value_type(code, text));
}

void PickOne::
finish_choices() {
  G_List_string items;

  Choices::const_iterator new_current = _choices.begin();
  
  Choices::const_iterator ci;
  for (ci = _choices.begin(); ci != _choices.end(); ++ci) {
    items.append((char *)(*ci).second.c_str());

    if (_has_item && (*ci).first == _current_item) {
      new_current = ci;
    }
  }
  set_popdown_strings(items);

  if (new_current != _choices.end()) {
    set_item(new_current);
  }
}

void PickOne::
remove_item(int item) {
  Choices::iterator ci;
  for (ci = _choices.begin(); ci != _choices.end(); ++ci) {
    if ((*ci).first == item) {
      remove_item(ci);
      return;
    }
  }
}

void PickOne::
remove_item(Choices::iterator ci) {
  if (_current_item == (*ci).first) {
    _has_item = false;
    get_entry()->set_text("");
  }

  _choices.erase(ci);
  finish_choices();
}

void PickOne::
set_item(int item) {
  Choices::const_iterator ci;
  for (ci = _choices.begin(); ci != _choices.end(); ++ci) {
    if ((*ci).first == item) {
      set_item(ci);
      return;
    }
  }
  _has_item = false;
}

void PickOne::
set_item(Choices::const_iterator ci) {
  _current_item = (*ci).first;
  _has_item = true;
  _internal_change = true;
  get_entry()->set_text((*ci).second);
  _internal_change = false;
}

bool PickOne::
has_item() {
  return _has_item;
}
 
int PickOne::
get_item(int default_value) {
  return _has_item ? _current_item : default_value;
}

void PickOne::
entry_changed() {
  if (_internal_change) {
    return;
  }
  string text = get_entry()->get_text();

  Choices::const_iterator ci;
  for (ci = _choices.begin(); ci != _choices.end(); ++ci) {
    if (text == (*ci).second) {
      if (!_has_item || _current_item != (*ci).first) {
	_current_item = (*ci).first;
	_has_item = true;
	changed_item();
      }
      return;
    }
  }

  if (_has_item) {
    _has_item = false;
    changed_item();
  }
}
