/* importList.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "importList.h"
#include "format.h"

ImportList::Row::
Row(const string &relpath, int type) : 
  _relpath(relpath), _type(type) 
{
}

ImportList::Column::
Column(const string &heading) : _heading(heading) {
  _type = CT_none;
}

void ImportList::Column::
set_type(ColumnType type) {
  _type = type;
  _heading = ImportList::get_type_description(type);
}

ImportList::
ImportList() {
}

int ImportList::
get_num_rows() const {
  return _rows.size();
}

int ImportList::
get_num_columns() const {
  return _columns.size();
}

void ImportList::
check_min_columns(int cols) {
  while (_columns.size() < cols) {
    _columns.push_back(format_int(_columns.size() + 1));
  }
}

Gtk_CList *ImportList::
make_clist() {
  assert(!_columns.empty());

  // First, get the good titles.
  vector<const char *> columns_cstr(_columns.size());
  int c;
  for (c = 0; c < _columns.size(); c++) {
    columns_cstr[c] = _columns[c]._heading.c_str();
  }

  // Create the CList,
  Gtk_CList *clist = new Gtk_CList(_columns.size(), &columns_cstr[0]);

  // And fill it up.
  int r;
  for (r = 0; r < _rows.size(); r++) {
    const ImportList::Row &row = _rows[r];
    for (c = 0; c < _columns.size(); c++) {
      if (c >= row._data.size()) {
	columns_cstr[c] = "";
      } else {
	columns_cstr[c] = row._data[c].c_str();
      }
    }
    clist->append(&columns_cstr[0]);
  }
  clist->columns_autosize();

  return clist;
}

string ImportList::
get_type_description(ColumnType type) {
  switch (type) {
  case CT_none:
    return "None";
    
  case CT_song_name:
    return "Song name";

  case CT_performer:
    return "Performer";

  case CT_writer:
    return "Writer";

  case CT_track_number:
    return "Track number";

  case CT_cd_title:
    return "CD title";

  case CT_cd_performer:
    return "CD performer";

  case CT_cd_writer:
    return "CD writer";
  }

  return "**Unknown**";
}

DataCell::Type ImportList::
get_type_datatype(ColumnType type) {
  switch (type) {
  case CT_none:
    return DataCell::DC_null;
    
  case CT_song_name:
  case CT_cd_title:
    return DataCell::DC_string;

  case CT_performer:
  case CT_writer:
  case CT_track_number:
  case CT_cd_performer:
  case CT_cd_writer:
    return DataCell::DC_int;
  }

  return DataCell::DC_string;
}
